#!/bin/bash

MEDIA_PATH=$1
NODES=()
scripts_found=false
media_found=false

shift
while [[ $# -gt 0 ]]
do
 NODES+=("$1")
 shift
done

cd $MEDIA_PATH

for file in *.sh; do
 if [ -e "$file" ]; then
  chmod +x $file
  if [ ${#NODES[*]} -gt 0 ]; then
   for node in ${NODES[@]}; do
    . $file $node >/dev/null 2>&1 &
    echo "Publishing script $file started for node $node"
   done
  else
   . $file >/dev/null 2>&1 &
   echo "Publishing script $file started"
  fi
  scripts_found=true
 fi
done

if ! $scripts_found; then
 for node in ${NODES[@]}; do
  for media_file in *.mp4; do
   stream_name=`echo $media_file | sed s/.mp4//g`
   stream_name="$node.$stream_name"
   ffmpeg -stream_loop -1 -re -y -rtbufsize 1k -i $media_file -preset ultrafast -ar 48000 -acodec aac -vcodec h264 -g 30 -strict -2 -f flv rtmp://$node:1935/live/$stream_name -nostdin -nostats </dev/null >/dev/null 2>&1 &
   echo "Stream $stream_name is published to node $node"
   media_found=true
  done
 done
fi

if ! $scripts_found && ! $media_found; then
 if [ ${#NODES[*]} -gt 0 ]; then
  echo "No scripts to run or media to publish in $MEDIA_PATH!"
 else
  echo "No nodes available to publish stream!"
 fi
 exit 1
fi

exit 0
